import os.path
from pathlib import Path

import click

from twitter_cleanup import TwitterCleanup


@click.group()
@click.option("--yes", "-y", help="Yes to all prompts", default=False, is_flag=True)
@click.pass_context
def cli(context, yes):
    """Twitter Cleanup"""
    context.obj = {"cleanup": TwitterCleanup(assume_yes=yes)}


@cli.command()
@click.argument("days", type=int)
@click.pass_context
def inactive(context, days):
    """Unfollow accounts inactive for a given amount of days."""
    context.obj["cleanup"].unfollow_inactive_for(days=days)


@cli.command()
@click.option(
    "--threshold",
    "-t",
    help="Botometer threshold (form 0 to 100, default=75)",
    default=75,
    type=click.IntRange(0, 100),
)
@click.pass_context
def bots(context, threshold):
    """Soft-block bots following you."""
    context.obj["cleanup"].soft_block_bots(threshold=threshold / 100)


@cli.command()
@click.pass_context
def clear_cache(context):
    """Clear cache files."""
    for file in Path(os.path.expanduser("~")).glob(".twitter-cleanup.cache.*"):
        click.echo(f"Deleting {file}")
        file.unlink()


if __name__ == "__main__":
    cli()



from enum import Flag

from botometer import Botometer, NoTimelineError

from twitter_cleanup.authentication import authentication


BotometerStatus = Flag("BotometerStatus", "PENDING READY UNAVAILABLE")


class BotometerResult:
    """Holds Botometer result and avoids repeating the request to their API"""

    def __init__(self):
        kwargs = authentication.botometer.copy()
        kwargs["wait_on_rate_limit"] = True
        self.botometer = Botometer(**kwargs)

        self.status = BotometerStatus.PENDING
        self._probability = None
        self.user_id = None

    def _get_result(self):
        try:
            result = self.botometer.check_account(self.user_id)
        except NoTimelineError:
            self.status = BotometerStatus.UNAVAILABLE
        else:
            self._probability = result.get("cap", {}).get("universal")
            self.status = BotometerStatus.READY

    @property
    def probability(self):
        if not self.user_id:
            raise RuntimeError("Cannot use Botometer without an account ID")

        if self.status is BotometerStatus.PENDING:
            self._get_result()
            return self.probability

        if self.status is BotometerStatus.UNAVAILABLE:
            return 0.0  # let's assume it's not a bot

        return self._probability



from os import getcwd

from decouple import AutoConfig
from tweepy import OAuthHandler


class Authentication:
    """Holds authentication data for further usage in the script"""

    def __init__(self):
        config = AutoConfig(getcwd())
        self.consumer_key = config("TWITTER_CONSUMER_KEY")
        self.consumer_secret = config("TWITTER_CONSUMER_SECRET")
        self.access_token = config("TWITTER_ACCESS_TOKEN_KEY")
        self.access_token_secret = config("TWITTER_ACCESS_TOKEN_SECRET")
        self.mashape_key = config("BOTOMETER_MASHAPE_KEY", default=None)

    @property
    def tweepy(self):
        """Returns an authentication object required by Tweepy"""
        auth = OAuthHandler(self.consumer_key, self.consumer_secret)
        auth.set_access_token(self.access_token, self.access_token_secret)
        return auth

    @property
    def botometer(self):
        """Returns a dictionary compatible with Botometer API"""
        return dict(
            consumer_key=self.consumer_key,
            consumer_secret=self.consumer_secret,
            access_token=self.access_token,
            access_token_secret=self.access_token_secret,
            mashape_key=self.mashape_key,
        )


authentication = Authentication()



import os.path
from datetime import datetime, timedelta
from pathlib import Path
from pickle import UnpicklingError, dump, load


class Cache:
    def __init__(self, called_by, caller_args, expires_in=7):
        self.expires_in = expires_in  # days
        self.called_by = called_by
        self.caller_args = caller_args
        file_name = f".twitter-cleanup.cache.{called_by}"
        self.cache = Path(os.path.expanduser("~")) / file_name

        if not self.cache.exists():
            self._create()

        if not self.is_valid():
            self.cache.unlink()
            self._create()

    def is_valid(self):
        assert self.cache.is_file()
        with open(self.cache, "rb") as fobj:
            try:
                data = load(fobj)
            except (UnpicklingError, EOFError):
                return False

            if not self.is_valid_data(data):
                return False

            limit = datetime.now() - timedelta(days=self.expires_in)
            expired = data["created_at"] <= limit
            if expired:
                return False

            return True

    def is_valid_data(self, data):
        keys = {"called_by", "caller_args", "created_at", "usernames"}
        if keys != data.keys():
            return False

        if data["called_by"] != self.called_by:
            return False

        if data["caller_args"] != self.caller_args:
            return False

        if not isinstance(data["created_at"], datetime):
            return False

        if data["usernames"]:
            for username, cached_user_data in data["usernames"].items():
                if not isinstance(username, str):
                    return False
                if not isinstance(cached_user_data, bool):
                    return False

        return True

    def _create(self):
        initial_data = {
            "called_by": self.called_by,
            "caller_args": self.caller_args,
            "created_at": datetime.now(),
            "usernames": dict(),
        }
        self._save(initial_data)

    def _read(self):
        assert self.is_valid()
        with open(self.cache, "rb") as fobj:
            data = load(fobj)
        return data

    def _save(self, data):
        assert self.is_valid_data(data)
        with open(self.cache, "wb") as fobj:
            dump(data, fobj)

    def set(self, username, cached_user_data):
        assert isinstance(cached_user_data, bool)
        data = self._read()
        assert self.is_valid_data(data)
        data["usernames"][username] = cached_user_data
        self._save(data)

    def get(self, username):
        data = self._read()
        return data["usernames"].get(username)

    def unset(self, username):
        data = self._read()
        if username in data["usernames"]:
            del data["usernames"][username]
        self._save(data)



import arrow
import click
from tweepy import API, Cursor

from twitter_cleanup.authentication import authentication
from twitter_cleanup.cache import Cache
from twitter_cleanup.user import User


class TwitterCleanup:
    """Core class of this package, holding the methods to clean up the
    authenticated Twitter account."""

    def __init__(self, assume_yes=False):
        self.api = API(authentication.tweepy, wait_on_rate_limit=True)
        self.me = self.api.me()
        self.assume_yes = assume_yes

    @property
    def following(self):
        """Generator with all accounts following the authenticated user"""
        for users in Cursor(self.api.friends).pages():
            yield from (User.parse(self.api, user._json) for user in users)

    @property
    def followers(self):
        """Generator with all accounts followed by the authenticated user"""
        for users in Cursor(self.api.followers).pages():
            yield from (User.parse(self.api, user._json) for user in users)

    def unfollow_inactive_for(self, **kwargs):
        """Takes any kwarg compatible with Python's `timedelta` and unfollows
        users whose last tweet are older than the `timedelta` defined by these
        kwargs"""
        total = self.me.friends_count
        to_unfollow = []
        cache = Cache("unfollow_inactive_for", kwargs)

        bar_kwargs = dict(
            length=total,
            label=f"Looking for inactive users in {total} accounts",
            width=0,  # 0 means full-width
        )
        with click.progressbar(**bar_kwargs) as bar:
            for user in self.following:
                should_unfollow = cache.get(user.screen_name)
                if isinstance(should_unfollow, bool):
                    should_unfollow = user.last_status_before(**kwargs)
                    cache.set(user.screen_name, should_unfollow)

                if should_unfollow:
                    to_unfollow.append(user)

                bar.update(1)

        for user in to_unfollow:
            self.unfollow(user)

    def unfollow(self, user):
        """Confirms and unfollow a given user"""
        last_tweet_date = arrow.get(user.status.created_at)
        message = (
            f"Confirm unfollow {user.screen_name}?\n\n"
            f"Last tweet was {last_tweet_date.humanize()}:\n\n"
            f"{user.status.text}\n\n"
        )
        if not self.confirm(message):
            return

        self.api.destroy_friendship(user.id)
        click.echo(f"Unfollowed {user.screen_name}")

    def soft_block_bots(self, threshold=None):
        """Soft-blocks every bot account classified by Botometer lower than the
        `threshold` (defaults to 0.75 in User class)."""
        total = self.me.followers_count
        to_block = []
        cache = Cache("soft_block_bots", threshold)

        bar_kwargs = dict(
            length=total,
            label=f"Looking for bots in {total} accounts",
            width=0,  # 0 means full-width
        )
        with click.progressbar(**bar_kwargs) as bar:
            for user in self.followers:
                should_soft_block = cache.get(user.screen_name)
                if isinstance(should_soft_block, bool):
                    should_soft_block = user.is_bot()
                    cache.set(user.screen_name, should_soft_block)

                if should_soft_block:
                    to_block.append(user)

                bar.update(1)

        for user in to_block:
            self.soft_block_bot(user)

    def soft_block_bot(self, user):
        """Confirms and soft-block a given account"""
        percent = 100 * user.botometer_result.probability
        message = (
            f"Confirm soft-block {user.screen_name}?\n"
            f"{percent:.2f}% probability of being a bot"
        )
        if not self.confirm(message):
            return

        self.api.create_block(user.id)
        self.api.destroy_block(user.id)
        click.echo(f"Soft-blocked {user.screen_name}")

    def confirm(self, message):
        if self.assume_yes:
            return True

        return click.confirm(message)



from setuptools import find_packages, setup


with open("README.rst") as fobj:
    long_description = fobj.read()


setup(
    author="Eduardo Cuducos",
    classifiers=[
        "Development Status :: 4 - Beta",
        "Intended Audience :: Developers",
        "Intended Audience :: End Users/Desktop",
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
        "Programming Language :: Python :: 3.6",
        "Topic :: Utilities",
    ],
    description="Clean-up iactive accounts and bots from your Twitter",
    entry_points="[console_scripts]\ntwitter-cleanup=twitter_cleanup.__main__:cli",
    install_requires=[
        "arrow>=0.13.0",
        "click>=7.0",
        "botometer>=1.3",
        "python-decouple>=3.1",
        "tweepy>=3.7.0",
    ],
    keywords="twitter, bots, social network",
    license="GPLv3",
    long_description=long_description,
    name="twitter-cleanup",
    packages=find_packages(),
    py_modules=["twitter_cleanup"],
    url="https://github.com/cuducos/twitter-cleanup",
    version="0.0.3",
    zip_safe=False,
)



from datetime import datetime, timedelta

from tweepy import models

from twitter_cleanup.botometer import BotometerResult


class User(models.User):
    """A wrapper for Tweepy native User model, including custom methods to
    check idle accounts and bots"""

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.botometer_result = BotometerResult()

    @classmethod
    def parse(cls, api, data):
        """This is how Tweepy instantiate User models from the API result"""
        return super(User, cls).parse(api, data)

    def last_status_before(self, **kwargs):
        """Takes any kwarg compatible with Python's `timedelta` and says
        whether the user's last tweet is older than the `timedelta` defined by
        these kwargs"""
        if not getattr(self, "status", None):
            return False

        return self.status.created_at < datetime.now() - timedelta(**kwargs)

    def is_bot(self, threshold=0.75):
        if self.protected:
            return False

        self.botometer_result.user_id = self.id
        return self.botometer_result.probability > threshold
